unit ComponentPropertyEx;

interface

uses
  classes, DsgnIntf;

type
  TComponentPropertyEx=class(TComponentProperty)
  public
    procedure Edit; override;
  end;

  TDatasetPropertyEx=class(TComponentProperty)
  private
    FCheckProc: TGetStrProc;
    procedure CheckComponent(const Value: string);
  public
    procedure GetValues(Proc: TGetStrProc); override;
    procedure Edit; override;
  end;

procedure Register;

implementation

uses
  DB, windows, forms, toolsapi;

procedure Register;
begin
  RegisterPropertyEditor( typeinfo(TComponent), nil, '', TComponentPropertyEx);
  RegisterPropertyEditor(TypeInfo(TDataSet), TDataSource, 'DataSet', TDataSetPropertyEx);
end;


function FocusControl(ed:TPropertyEditor):boolean;
 var comp,own:TComponent;frm:TCustomForm; frmdsg:IFormDesigner;
 ImplFileName, IntfFileName, FormFileName: string;
begin
  {inherited; ignore}

  comp := TObject(TComponentPropertyEx(ed).GetOrdValue) as TComponent;
  if assigned(comp) then
  begin
    {Get components final owner}
    own := comp;
    while assigned(own.owner) do
      own := own.owner;

    frm := own as TCustomForm;
    Assert(assigned(frm.Designer));
    frmdsg := (frm.Designer as IFormDesigner);

    {Clear Previous selections}
    ed.Designer.SelectComponent(nil);

    {Make sure that the source file is open}
    frmdsg.ModuleFileNames(ImplFilename, IntfFilename, FormFileName);
    (BorlandIDEServices as IOTAActionServices).OpenFile(ImplFileName);

    {Make the form visible
    frm.Show;
    frm.SetFocus;
    frm.BringToFront;}

    {Select new component}
    frmdsg.SelectComponent(comp);

    result := true;
  end

  else {! assigned}
  begin
    MessageBeep(0);
    result := false;
  end;
end;

{ TComponentPropertyEditor }

procedure TComponentPropertyEx.Edit;
begin
  {inherited; ignore}
  FocusControl(self);
end;

{ TDatasetPropertyEx }

procedure TDatasetPropertyEx.CheckComponent(const Value: string);
var
  J: Integer;
  Dataset: TDataset;
begin
  Dataset := TDataset(Designer.GetComponent(Value));
  for J := 0 to PropCount - 1 do
    if TDataSource(GetComponent(J)).IsLinkedTo(Dataset) then
      Exit;
  FCheckProc(Value);
end;

procedure TDatasetPropertyEx.Edit;
begin
  {inherited; ignore}
  FocusControl(self);
end;

procedure TDatasetPropertyEx.GetValues(Proc: TGetStrProc);
begin
  FCheckProc := Proc;
  inherited GetValues(CheckComponent);
end;

end.
